rm(list=ls())

setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/fact_analysis/pfa")

beer.data <- read.table(file="beer_rnd.txt",sep="\t",header=T)
print(head(beer.data,n=5))

#********** PRINCIPAL COMPONENT ANALYSIS ***********

#calculation of the correlation matrix
beer.cor <- cor(beer.data)
print(round(beer.cor,2))

#calculation of the eigenvalues and eigenvectors
eig.pca <- eigen(beer.cor)

#print eigenvalues
print("eigenvalues")
print(eig.pca$values)

#screeplot
plot(1:14,eig.pca$values,type="b")
abline(a=1,b=0)

#loadings
loadings.pca <- matrix(0,nrow=nrow(beer.cor),ncol=3)
for (j in 1:3){
  loadings.pca[,j] <- sqrt(eig.pca$values[j])*eig.pca$vectors[,j]
}
print("loadings for the 3 first factors")
rownames(loadings.pca) <- colnames(beer.data)
print(round(loadings.pca,5))

#communalities for the three first factors
comm.pca <- apply(loadings.pca,1,function(x){sum(x^2)})
print("communalities for the 3 first factors")
names(comm.pca) <- colnames(beer.data)
print(round(comm.pca,5))

#computation of the factor scores

#the variance of the scores is 1 for each factor (SAS, SPSS)
print("factor scores - 1st version")
#inverse of the correlation matrix
inv.beer.cor <- solve(beer.cor)
fscores.pca <- inv.beer.cor%*%loadings.pca
print(round(fscores.pca,5))

#the variance of the scores is equal to the eigenvalue for each factor [TANAGRA (1/n), R princomp (1/n), R prcomp (1/n-1), R FactoMineR (1/n)...]
print("factor scores - 2nd version")
for (j in 1:3){
  fscores.pca[,j] <- sqrt(eig.pca$values[j])*fscores.pca[,j]
}
print(round(fscores.pca,7))

#********** PRINCIPAL FACTOR ANALYSIS (non iterative) *****

#uniqueness
d2 <- 1/diag(inv.beer.cor)
print(d2)
#prior communalities
init.comm <- 1-d2
print(init.comm)

#the new matrix to diagonalize
beer.cor.pfa <- beer.cor
diag(beer.cor.pfa) <- init.comm
print("matrix to diagonalize")
print(round(beer.cor.pfa,2))
print("trace of the matrix")
print(sum(diag(beer.cor.pfa)))

#eigenvalues
eig.pfa <- eigen(beer.cor.pfa)
print("eigenvalues")
print(eig.pfa$values)
#screeplot
plot(1:14,eig.pfa$values,type="b")
#loadings
loadings.pfa <- matrix(0,nrow=nrow(beer.cor.pfa),ncol=2)
for (j in 1:2){
  loadings.pfa[,j] <- sqrt(abs(eig.pfa$values[j]))*eig.pfa$vectors[,j]
}
#print
print("loadings for the 2 first factors")
rownames(loadings.pfa) <- colnames(beer.data)
print(round(loadings.pfa,5))
print("communalities for the 2 first factors")
#prior and estimated communalities for the 2 first factors
comm.pfa <- apply(loadings.pfa,1,function(x){sum(x^2)})
names(comm.pfa) <- colnames(beer.data)
print(round(cbind(init.comm,comm.pfa),5))
#factor scores
print("factor scores")
fscores.pfa <- inv.beer.cor%*%loadings.pfa
print(round(fscores.pfa,5))
#variance of the scores
vscores <- numeric(2)
for (j in 1:2){
 vscores[j] <- sum(fscores.pfa[,j]*loadings.pfa[,j])
}
print(round(vscores,5))

#******** HARRIS Component Analysis (SAS) ou Image (Kaiser, 1963) (SPSS)
S <- matrix(0,nrow=nrow(beer.cor),ncol=ncol(beer.cor))
diag(S) <- sqrt(1/diag(inv.beer.cor))
inv.S <- solve(S)
beer.cor.harris <- beer.cor
diag(beer.cor.harris) <- init.comm
beer.cor.harris <- inv.S%*%beer.cor.harris%*%inv.S
print("matrix to diagonalize")
print(round(beer.cor.harris,2))
print("trace of the matrix")
print(sum(diag(beer.cor.harris)))

#valeurs et vecteurs propres
eig.harris <- eigen(beer.cor.harris)
print("eigenvalues")
print(eig.harris$values)

plot(1:14,eig.harris$values,type="b")

#loadings
loadings.harris <- matrix(0,nrow=nrow(beer.cor.harris),ncol=2)
for (j in 1:2){
  loadings.harris[,j] <- sqrt(eig.harris$values[j])*eig.harris$vectors[,j]*sqrt(d2)
}
print("loadings for the 2 first factors")
rownames(loadings.harris) <- colnames(beer.data)
print(round(loadings.harris,5))

#unweighted variance explained
unweighted.var.harris <- apply(loadings.harris,2,function(x){sum(x^2)})
print(round(unweighted.var.harris,5))

#communalits
print("communalities for the 2 first factors")
comm.harris <- apply(loadings.harris,1,function(x){sum(x^2)})
print(round(cbind(init.comm,comm.harris),5))

#unweighted variance explained
print("unweighted variance explained")
var.harris <- apply(loadings.harris,2,function(x){sum(x^2)})
print(var.harris)

#factor scores
print("factor scores")
fscores.harris <- inv.beer.cor%*%loadings.harris
print(round(fscores.harris,5))

#variance of the scores
vscores.harris <- numeric(2)
for (j in 1:2){
 vscores.harris[j] <- sum(fscores.harris[,j]*loadings.harris[,j])
}
print(round(vscores.harris,5))

#********** USING PSYCH PACKAGE ****************

#load the libraries
library(psych)
library(GPArotation)

#PCA
pca.unrotated <- principal(beer.data, nfactors=2, rotate="none")
print(pca.unrotated$loadings[,])

#PCA + varimax
pca.rotated <- principal(beer.data, nfactors=2, rotate="varimax")
print(pca.rotated$loadings[,])

#PFA (principal factor analysis)
pfa.unrotated <- fa(beer.data,nfactors=2,rotate="none",SMC=T,fm="pa",max.iter=1)
print(pfa.unrotated$loadings[,])

#PFA + varimax
pfa.varimax <- fa(beer.data,nfactors=2,rotate="varimax",SMC=T,fm="pa",max.iter=1)
print(pfa.varimax$loadings[,])

#********** FIN ****************